local ball, paddle

local function reflectBall(ball, surfaceNormalX, surfaceNormalY)

    -- Calculate the dot product of ball speed and surface normal
    local dotProduct = ball.vx * surfaceNormalX + ball.vy * surfaceNormalY

    -- Reverse direction of ball speed based on surface normal
    ball.vx = ball.vx - 2 * dotProduct * surfaceNormalX
    ball.vy = ball.vy - 2 * dotProduct * surfaceNormalY

    -- Calculate the magnitude of the speed of the ball (its length)
    local mag = math.sqrt(ball.vx^2 + ball.vy^2)

    -- Normalize ball speed to avoid infinite bounces
    if mag > 0 then --check if the magnitude is not zero (which avoids divisions by zero)
        ball.vx = ball.vx / mag -- divide the horizontal component of the velocity by the magnitude
        ball.vy = ball.vy / mag -- divide the vertical component of the velocity by the magnitude
    end

end

--[[ Slightly otpimized

local function reflectBall(ball, surfaceNormalX, surfaceNormalY)
    local dotProduct = ball.vx * surfaceNormalX + ball.vy * surfaceNormalY
    ball.vx = ball.vx - 2 * dotProduct * surfaceNormalX
    ball.vy = ball.vy - 2 * dotProduct * surfaceNormalY
    local magSq = ball.vx^2 + ball.vy^2
    if magSq > 0 then
        local invMag = 1 / math.sqrt(magSq)
        ball.vx = ball.vx * invSpeed
        ball.vy = ball.vy * invSpeed
    end
end

]]


function love.load()

    math.randomseed(os.time())

    local a = math.random() * (math.pi*2)
    local vx = math.cos(a)
    local vy = math.sin(a)

    ball = {x = 100, y = 100, radius = 10, vx = vx, vy = vy}
    paddle = {x = love.graphics.getWidth() / 2, y = love.graphics.getHeight() - 20, width = 80, height = 10}

end

function love.update(dt)

    -- Update the position of the ball based on its speed

    ball.x = ball.x + ball.vx * 400 * dt
    ball.y = ball.y + ball.vy * 400 * dt

    -- Check for collisions with screen edges

    if ball.x - ball.radius < 0 then
        ball.vx = math.abs(ball.vx)
    elseif ball.x + ball.radius > love.graphics.getWidth() then
        ball.vx = -math.abs(ball.vx)
    end

    if ball.y - ball.radius < 0 then
        ball.vy = math.abs(ball.vy)
    elseif ball.y + ball.radius > love.graphics.getHeight() then
        ball.vy = -math.abs(ball.vy)
    end

    -- Check for collisions with the racket

    if ball.y + ball.radius > paddle.y and ball.x > paddle.x and ball.x < paddle.x + paddle.width then

        -- Calculate the surface normal based on where the ball hit the racket
        local surfaceNormalX = (ball.x - (paddle.x + paddle.width / 2)) / (paddle.width / 2)
        reflectBall(ball, surfaceNormalX, -1)

        -- Replacement of the ball to avoid it being glitched in the racket
        ball.y = paddle.y - ball.radius

    end

end

function love.mousemoved(x, y)
    paddle.x = x - paddle.width / 2
end

function love.draw()
    love.graphics.circle("fill", ball.x, ball.y, ball.radius)
    love.graphics.rectangle("fill", paddle.x, paddle.y, paddle.width, paddle.height)
end
